-- Cette ligne permet d'afficher des traces dans la console pendant l'éxécution
io.stdout:setvbuf('no')

-- Empèche Love de filtrer les contours des images quand elles sont redimentionnées
-- Indispensable pour du pixel art
love.graphics.setDefaultFilter("nearest")

-- Cette ligne permet de déboguer pas à pas dans ZeroBraneStudio
if arg[#arg] == "-debug" then require("mobdebug").start() end

function love.load()
  
  largeurEcran = love.graphics.getWidth()
  hauteurEcran = love.graphics.getHeight()
  
  isAlive = true
  
  --State Management
  menu = "main"
  mainMenu = true
  
  background = love.graphics.newImage("src/img/background.png")
  gameOver = love.graphics.newImage("src/img/gameover.png")
  
  --timers
  --Aliens
  createAlienTimerMax = 1
  createAlienTimer = createAlienTimerMax
  createAlienTimerReduce = 0.5
  --Shots
  createShotTimerMax = 1
  createShotTimer = createShotTimerMax
  createShotTimerReduce = 3
  
  --table player
  ship = {}
  ship.img = love.graphics.newImage("src/img/spaceship.png")
  ship.x = largeurEcran/2 - ship.img:getWidth()/2
  ship.y = hauteurEcran - ship.img:getHeight() - 25
  ship.speed = 200
  ship.canShoot = false
  ship.shots = {}
  shotImg = love.graphics.newImage("src/img/spaceship_shot.png")
  
  player = {}
  player.score = 0
  
  playImgOn = love.graphics.newImage("src/img/play_on.png")
  playImgOff = love.graphics.newImage("src/img/play_off.png")
  exitImgOn = love.graphics.newImage("src/img/exit_on.png")
  exitImgOff = love.graphics.newImage("src/img/exit_off.png")
  
  menu = {}
  menu.play = {}
  menu.play.img = playImgOff
  menu.play.x = largeurEcran/2 - menu.play.img:getWidth() -20
  menu.play.y = hauteurEcran/2 - menu.play.img:getHeight()
  menu.exit = {}
  menu.exit.img = exitImgOff
  menu.exit.x = largeurEcran/2 + 20
  menu.exit.y = hauteurEcran/2 - menu.exit.img:getHeight() 
  
  aliens = {}
  alienImg = love.graphics.newImage("src/img/spaceship_alien.png")
  
end

--Collision needs x, y and img
function collision(shot, enemy)
  if shot.x < enemy.x + enemy.img:getWidth() and 
     enemy.x < shot.x + shot.img:getWidth() and
     shot.y < enemy.y + enemy.img:getHeight() and
     enemy.y < shot.y + shot.img:getHeight() then
    return true
  else
    return false
  end
end

function love.update(dt)
  
  --Shot Management
  --Timer
  createShotTimer = createShotTimer - (createShotTimerReduce * dt)
  --Move
  for i, shot in ipairs(ship.shots) do
    shot.y = shot.y - shot.speed * dt
    --Shot can't get out of the screen
    if shot.y < -10 then
      table.remove(ship.shots, i)
    end
  end
  
  
  if menu == "level" then
  
    --Timers Management
    createAlienTimer = createAlienTimer - (createAlienTimerReduce * dt)
    
    --Create Aliens with Timers
    if createAlienTimer < 0 then
      createAlienTimer = createAlienTimerMax
      randomX = math.random(20, largeurEcran - 52)
      newAlien = { x = randomX, y = -52, img = alienImg, speed = 100}
      table.insert(aliens, newAlien)
    end
    
    --Actions for each aliens
    for i, alien in ipairs(aliens) do
      --Move
      alien.y = alien.y + alien.speed * dt
      --Remove when out of the screen
      if alien.y > hauteurEcran + 10 then
        table.remove(aliens, i)
      end
      if collision(alien, ship) == true then
        isAlive = false
      end
    end
    
    --Actions for each shot
    for i, shot in ipairs(ship.shots) do
      -- Collision Shot - Alien
      for j, alien in ipairs(aliens) do
        if collision(shot, alien) then
          table.remove(aliens, j)
          table.remove(ship.shots, i)
          player.score = player.score + 10
        end
      end
    end
    
    if isAlive == false then
      menu = "gameOver"
    end
  
  end

  if mainMenu == true then
    --Focus the button when player is at the same x
    if ship.x + ship.img:getWidth() > menu.play.x and ship.x + ship.img:getWidth() < menu.play.x + menu.play.img:getWidth()then
      menu.play.img = playImgOn
    elseif ship.x + ship.img:getWidth() > menu.exit.x and ship.x + ship.img:getWidth() < menu.exit.x + menu.exit.img:getWidth() then
      menu.exit.img = exitImgOn
    else
      menu.exit.img = exitImgOff
      menu.play.img = playImgOff
    end
    --Collision shot - buttons
    for i, shot in ipairs(ship.shots) do
      if collision(shot, menu.play) == true then
        table.remove(ship.shots, i)
        mainMenu = false
        menu = "level"
      elseif collision(shot, menu.exit) == true then
        love.event.quit()
      end
    end
  end
  
  --Keyboard
  --Shoot
  if love.keyboard.isDown("space") then
    if createShotTimer < 0 then
      createShotTimer = createShotTimerMax
      ship.canShoot = true
    else
      ship.canShoot = false
    end
    if ship.canShoot == true then
      newShot = {x = ship.x + ship.img:getWidth()/2, y = ship.y, img = shotImg, speed = 100}
      table.insert(ship.shots, newShot)
    end
    --if menu == "gameOver" then
    --  mainMenu = true
    --end
  end
  --Move left
  if love.keyboard.isDown("left") then
    ship.x = ship.x - ship.speed * dt
    --Can't get out of the screen
    if ship.x < 0 then
      ship.x = 0
    end
  end
  --Move right
  if love.keyboard.isDown("right") then
    ship.x = ship.x + ship.speed * dt
    --Can't get out of the screen
    if ship.x > largeurEcran - ship.img:getWidth() then
      ship.x = largeurEcran - ship.img:getWidth()
    end
  end
  
end

function love.draw()
  
  love.graphics.draw(background, 0, 0)
  
  --Draw main menu
  if mainMenu == true then
    love.graphics.draw(menu.play.img, menu.play.x, menu.play.y)
    love.graphics.draw(menu.exit.img, menu.exit.x, menu.exit.y)
  end
  
  --Draw level and elements in mainMenu
  if mainMenu == true or menu == "level" then
    --Draw ship
    if isAlive == true then
    love.graphics.draw(ship.img, ship.x, ship.y)
    end
    
    --Draw shots
    for i, shot in ipairs(ship.shots) do
      love.graphics.draw(shot.img, shot.x, shot.y)
    end
  end

  --Draw level
  if menu == "level" then
    --Draw aliens
    for i, alien in ipairs(aliens) do
      love.graphics.draw(alien.img, alien.x, alien.y)
    end
    --Print score
    love.graphics.print("Score : ".. player.score, largeurEcran - 100, 20)
  end
  
  if menu == "gameOver" then
    love.graphics.draw(gameOver, 0, 0)
    love.graphics.print("Score :".. player.score, largeurEcran/2 - 50, hauteurEcran/3)
    love.graphics.print("Press Escape to quit !", largeurEcran/2 - 75, hauteurEcran/3 + 50)
  end
  
end

function love.keypressed(key)
  
  if key == "escape" then
    love.event.quit()
  end
  
  print(key)
  
end